import 'package:health/health.dart';

import 'package:vivanta_connect_flutter/enums/vivanta_activity_type.dart';
import 'package:vivanta_connect_flutter/enums/vivanta_parameter_type.dart';
import 'package:vivanta_connect_flutter/helpers/helpers.dart';

class Healthkit {
  HealthFactory health = HealthFactory(useHealthConnectIfAvailable: true);

  Future<bool> requestAuthorization(
      List<HealthPermission> healthPermissions) async {
    final types =
        healthPermissions.map<HealthDataType>((e) => e.healthDataType).toList();
    final permissions = healthPermissions
        .map<HealthDataAccess>((e) => e.healthDataAccess)
        .toList();
    return health.requestAuthorization(types, permissions: permissions);
  }

  Future<bool> authorizationStatus(HealthDataType dataType) async {
    return health.isDataTypeAvailable(dataType);
  }

  Future<List<HealthDataPoint>> getDataPoint(
    List<HealthDataType> types,
    DateTime start,
    DateTime end,
  ) async {
    List<HealthDataPoint> healthData = await health.getHealthDataFromTypes(
      start,
      end,
      types,
    );

    return healthData;
  }

  Future<int> getTotalSteps(DateTime start, DateTime end) async {
    int? steps = await health.getTotalStepsInInterval(start, end);
    return steps ?? 0;
  }

  Future<bool> writeParameter({
    required VivantaParameterType type,
    required DateTime start,
    required DateTime end,
    required double value,
  }) async {
    final healtDataType = _getDataTypeFromVivantaParameter(type);
    return health.writeHealthData(value, healtDataType, start, end);
  }

  Future<bool> writeActivity({
    required VivantaActivityType type,
    required DateTime start,
    required DateTime end,
    int? calories,
    int? distance,
  }) {
    final activityType = _getWorkoutFromVivantaActivity(type);
    return health.writeWorkoutData(
      activityType,
      start,
      end,
      totalEnergyBurned: calories,
      totalDistance: distance,
    );
  }

  Future<bool> writeBloodPressure({
    required int systolic,
    required int diastolic,
    required DateTime start,
    required DateTime end,
  }) {
    return health.writeBloodPressure(systolic, diastolic, start, end);
  }

  Future<bool> writeBloodOxygen({
    required double saturation,
    required DateTime start,
    required DateTime end,
    double flowRate = 0.0,
  }) {
    return health.writeBloodOxygen(saturation, start, end, flowRate: flowRate);
  }

  Future<bool> writeAudiogram({
    required List<double> frequencies,
    required List<double> leftEarSensitivities,
    required List<double> rightEarSensitivities,
    required DateTime start,
    required DateTime end,
  }) {
    return health.writeAudiogram(
      frequencies,
      leftEarSensitivities,
      rightEarSensitivities,
      start,
      end,
    );
  }

  HealthDataType _getDataTypeFromVivantaParameter(VivantaParameterType type) {
    switch (type) {
      case (VivantaParameterType.activeCalories):
        return HealthDataType.ACTIVE_ENERGY_BURNED;
      case (VivantaParameterType.basalCalories):
        return HealthDataType.BASAL_ENERGY_BURNED;
      case (VivantaParameterType.bloodGlucose):
        return HealthDataType.BLOOD_GLUCOSE;
      case (VivantaParameterType.bloodOxygen):
        return HealthDataType.BLOOD_OXYGEN;
      case (VivantaParameterType.bloodPressureSystloc):
        return HealthDataType.BLOOD_PRESSURE_SYSTOLIC;
      case (VivantaParameterType.bloodPressureDiastolic):
        return HealthDataType.BLOOD_PRESSURE_DIASTOLIC;
      case (VivantaParameterType.bodyFatPercentage):
        return HealthDataType.BODY_FAT_PERCENTAGE;
      case (VivantaParameterType.bmi):
        return HealthDataType.BODY_MASS_INDEX;
      case (VivantaParameterType.bodyTemperature):
        return HealthDataType.BODY_TEMPERATURE;
      case (VivantaParameterType.heartRate):
        return HealthDataType.HEART_RATE;
      case (VivantaParameterType.height):
        return HealthDataType.HEIGHT;
      case (VivantaParameterType.restingHeartRate):
        return HealthDataType.RESTING_HEART_RATE;
      case (VivantaParameterType.respiratoryRate):
        return HealthDataType.RESPIRATORY_RATE;
      case (VivantaParameterType.steps):
        return HealthDataType.STEPS;
      case (VivantaParameterType.weight):
        return HealthDataType.WEIGHT;
      case (VivantaParameterType.flightsClimbed):
        return HealthDataType.FLIGHTS_CLIMBED;
      case (VivantaParameterType.totalSleepDuration):
        return HealthDataType.SLEEP_ASLEEP;
      case (VivantaParameterType.lightSleepDuration):
        return HealthDataType.SLEEP_LIGHT;
      case (VivantaParameterType.deepSleepDuration):
        return HealthDataType.SLEEP_DEEP;
      case (VivantaParameterType.remSleepDuration):
        return HealthDataType.SLEEP_REM;
      case (VivantaParameterType.sleepInBedDuration):
        return HealthDataType.SLEEP_IN_BED;
      case (VivantaParameterType.water):
        return HealthDataType.WATER;
      case (VivantaParameterType.electrodermalActivity):
        return HealthDataType.ELECTRODERMAL_ACTIVITY;
      case (VivantaParameterType.peripheralPerfusionIndex):
        return HealthDataType.PERIPHERAL_PERFUSION_INDEX;
      case (VivantaParameterType.waistCircumference):
        return HealthDataType.WAIST_CIRCUMFERENCE;
      case (VivantaParameterType.walkingHeartRate):
        return HealthDataType.WALKING_HEART_RATE;
      case (VivantaParameterType.distance):
        return HealthDataType.DISTANCE_WALKING_RUNNING;
      case (VivantaParameterType.mindfulness):
        return HealthDataType.MINDFULNESS;
      case (VivantaParameterType.activityTime):
        return HealthDataType.EXERCISE_TIME;
      case (VivantaParameterType.headacheNotPresent):
        return HealthDataType.HEADACHE_NOT_PRESENT;
      case (VivantaParameterType.headacheMild):
        return HealthDataType.HEADACHE_MILD;
      case (VivantaParameterType.headacheModerate):
        return HealthDataType.HEADACHE_MODERATE;
      case (VivantaParameterType.headacheSevere):
        return HealthDataType.HEADACHE_SEVERE;
      case (VivantaParameterType.headacheUnspecified):
        return HealthDataType.HEADACHE_UNSPECIFIED;
      default:
        return HealthDataType.STEPS;
    }
  }

  HealthWorkoutActivityType _getWorkoutFromVivantaActivity(
    VivantaActivityType activity,
  ) {
    switch (activity) {
      case VivantaActivityType.archery:
        return HealthWorkoutActivityType.ARCHERY;
      case VivantaActivityType.badminton:
        return HealthWorkoutActivityType.BADMINTON;
      case VivantaActivityType.baseball:
        return HealthWorkoutActivityType.BASEBALL;
      case VivantaActivityType.basketball:
        return HealthWorkoutActivityType.BASKETBALL;
      case VivantaActivityType.biking:
        return HealthWorkoutActivityType.BIKING;
      case VivantaActivityType.boxing:
        return HealthWorkoutActivityType.BOXING;
      case VivantaActivityType.cricket:
        return HealthWorkoutActivityType.CRICKET;
      case VivantaActivityType.curling:
        return HealthWorkoutActivityType.CURLING;
      case VivantaActivityType.elliptical:
        return HealthWorkoutActivityType.ELLIPTICAL;
      case VivantaActivityType.fencing:
        return HealthWorkoutActivityType.FENCING;
      case VivantaActivityType.americanFootball:
        return HealthWorkoutActivityType.AMERICAN_FOOTBALL;
      case VivantaActivityType.australianFootball:
        return HealthWorkoutActivityType.AUSTRALIAN_FOOTBALL;
      case VivantaActivityType.soccer:
        return HealthWorkoutActivityType.SOCCER;
      case VivantaActivityType.golf:
        return HealthWorkoutActivityType.GOLF;
      case VivantaActivityType.gymnastics:
        return HealthWorkoutActivityType.GYMNASTICS;
      case VivantaActivityType.handball:
        return HealthWorkoutActivityType.HANDBALL;
      case VivantaActivityType.highIntensityIntervalTraining:
        return HealthWorkoutActivityType.HIGH_INTENSITY_INTERVAL_TRAINING;
      case VivantaActivityType.hiking:
        return HealthWorkoutActivityType.HIKING;
      case VivantaActivityType.hockey:
        return HealthWorkoutActivityType.HOCKEY;
      case VivantaActivityType.skating:
        return HealthWorkoutActivityType.SKATING;
      case VivantaActivityType.jumpRope:
        return HealthWorkoutActivityType.JUMP_ROPE;
      case VivantaActivityType.kickboxing:
        return HealthWorkoutActivityType.KICKBOXING;
      case VivantaActivityType.martialArts:
        return HealthWorkoutActivityType.MARTIAL_ARTS;
      case VivantaActivityType.pilates:
        return HealthWorkoutActivityType.PILATES;
      case VivantaActivityType.racquetball:
        return HealthWorkoutActivityType.RACQUETBALL;
      case VivantaActivityType.rowing:
        return HealthWorkoutActivityType.ROWING;
      case VivantaActivityType.rugby:
        return HealthWorkoutActivityType.RUGBY;
      case VivantaActivityType.running:
        return HealthWorkoutActivityType.RUNNING;
      case VivantaActivityType.sailing:
        return HealthWorkoutActivityType.SAILING;
      case VivantaActivityType.crossCountrySkiing:
        return HealthWorkoutActivityType.CROSS_COUNTRY_SKIING;
      case VivantaActivityType.downhillSkiing:
        return HealthWorkoutActivityType.DOWNHILL_SKIING;
      case VivantaActivityType.snowboarding:
        return HealthWorkoutActivityType.SNOWBOARDING;
      case VivantaActivityType.softball:
        return HealthWorkoutActivityType.SOFTBALL;
      case VivantaActivityType.squash:
        return HealthWorkoutActivityType.SQUASH;
      case VivantaActivityType.stairClimbing:
        return HealthWorkoutActivityType.STAIR_CLIMBING;
      case VivantaActivityType.swimming:
        return HealthWorkoutActivityType.SWIMMING;
      case VivantaActivityType.tableTennis:
        return HealthWorkoutActivityType.TABLE_TENNIS;
      case VivantaActivityType.tennis:
        return HealthWorkoutActivityType.TENNIS;
      case VivantaActivityType.volleyball:
        return HealthWorkoutActivityType.VOLLEYBALL;
      case VivantaActivityType.walking:
        return HealthWorkoutActivityType.WALKING;
      case VivantaActivityType.waterPolo:
        return HealthWorkoutActivityType.WATER_POLO;
      case VivantaActivityType.yoga:
        return HealthWorkoutActivityType.YOGA;
      case VivantaActivityType.bowling:
        return HealthWorkoutActivityType.BOWLING;
      case VivantaActivityType.crossTraining:
        return HealthWorkoutActivityType.CROSS_TRAINING;
      case VivantaActivityType.trackAndField:
        return HealthWorkoutActivityType.TRACK_AND_FIELD;
      case VivantaActivityType.discSports:
        return HealthWorkoutActivityType.DISC_SPORTS;
      case VivantaActivityType.lacrosse:
        return HealthWorkoutActivityType.LACROSSE;
      case VivantaActivityType.preparationAndRecovery:
        return HealthWorkoutActivityType.PREPARATION_AND_RECOVERY;
      case VivantaActivityType.flexibility:
        return HealthWorkoutActivityType.FLEXIBILITY;
      case VivantaActivityType.cooldown:
        return HealthWorkoutActivityType.COOLDOWN;
      case VivantaActivityType.wheelchairWalkPace:
        return HealthWorkoutActivityType.WHEELCHAIR_WALK_PACE;
      case VivantaActivityType.wheelchairRunPace:
        return HealthWorkoutActivityType.WHEELCHAIR_RUN_PACE;
      case VivantaActivityType.handCycling:
        return HealthWorkoutActivityType.HAND_CYCLING;
      case VivantaActivityType.coreTraining:
        return HealthWorkoutActivityType.CORE_TRAINING;
      case VivantaActivityType.functionalStrengthTraining:
        return HealthWorkoutActivityType.FUNCTIONAL_STRENGTH_TRAINING;
      case VivantaActivityType.traditionalStrengthTraining:
        return HealthWorkoutActivityType.TRADITIONAL_STRENGTH_TRAINING;
      case VivantaActivityType.mixedCardio:
        return HealthWorkoutActivityType.MIXED_CARDIO;
      case VivantaActivityType.stairs:
        return HealthWorkoutActivityType.STAIRS;
      case VivantaActivityType.stepTraining:
        return HealthWorkoutActivityType.STEP_TRAINING;
      case VivantaActivityType.fitnessGaming:
        return HealthWorkoutActivityType.FITNESS_GAMING;
      case VivantaActivityType.barre:
        return HealthWorkoutActivityType.BARRE;
      case VivantaActivityType.cardioDance:
        return HealthWorkoutActivityType.CARDIO_DANCE;
      case VivantaActivityType.socialDance:
        return HealthWorkoutActivityType.SOCIAL_DANCE;
      case VivantaActivityType.mindAndBody:
        return HealthWorkoutActivityType.MIND_AND_BODY;
      case VivantaActivityType.pickleball:
        return HealthWorkoutActivityType.PICKLEBALL;
      case VivantaActivityType.climbing:
        return HealthWorkoutActivityType.CLIMBING;
      case VivantaActivityType.equestrianSports:
        return HealthWorkoutActivityType.EQUESTRIAN_SPORTS;
      case VivantaActivityType.fishing:
        return HealthWorkoutActivityType.FISHING;
      case VivantaActivityType.hunting:
        return HealthWorkoutActivityType.HUNTING;
      case VivantaActivityType.play:
        return HealthWorkoutActivityType.PLAY;
      case VivantaActivityType.snowSports:
        return HealthWorkoutActivityType.SNOW_SPORTS;
      case VivantaActivityType.paddleSports:
        return HealthWorkoutActivityType.PADDLE_SPORTS;
      case VivantaActivityType.surfingSports:
        return HealthWorkoutActivityType.SURFING_SPORTS;
      case VivantaActivityType.waterFitness:
        return HealthWorkoutActivityType.WATER_FITNESS;
      case VivantaActivityType.waterSports:
        return HealthWorkoutActivityType.WATER_SPORTS;
      case VivantaActivityType.taiChi:
        return HealthWorkoutActivityType.TAI_CHI;
      case VivantaActivityType.wrestling:
        return HealthWorkoutActivityType.WRESTLING;
    }
  }
}
